@@ -8,6 +8,7 @@
 #include "src/base/optional.h"
 #include "src/codegen/code-stub-assembler.h"
 #include "src/compiler/code-assembler.h"
+#include "src/objects/dictionary.h"
 
 namespace v8 {
 namespace internal {
@@ -34,15 +35,17 @@ class V8_EXPORT_PRIVATE AccessorAssembler : public CodeStubAssembler {
   void GenerateLoadSuperIC();
   void GenerateLoadSuperICBaseline();
   void GenerateKeyedLoadIC();
+  void GenerateEnumeratedKeyedLoadIC();
   void GenerateKeyedLoadIC_Megamorphic();
-  void GenerateKeyedLoadIC_MegamorphicStringKey();
   void GenerateKeyedLoadIC_PolymorphicName();
   void GenerateKeyedLoadICTrampoline();
   void GenerateKeyedLoadICBaseline();
+  void GenerateEnumeratedKeyedLoadICBaseline();
   void GenerateKeyedLoadICTrampoline_Megamorphic();
-  void GenerateKeyedLoadICTrampoline_MegamorphicStringKey();
   void GenerateStoreIC();
+  void GenerateStoreIC_Megamorphic();
   void GenerateStoreICTrampoline();
+  void GenerateStoreICTrampoline_Megamorphic();
   void GenerateStoreICBaseline();
   void GenerateDefineNamedOwnIC();
   void GenerateDefineNamedOwnICTrampoline();
@@ -69,6 +72,7 @@ class V8_EXPORT_PRIVATE AccessorAssembler : public CodeStubAssembler {
 
   void GenerateKeyedStoreIC();
   void GenerateKeyedStoreICTrampoline();
+  void GenerateKeyedStoreICTrampoline_Megamorphic();
   void GenerateKeyedStoreICBaseline();
 
   void GenerateDefineKeyedOwnIC();
@@ -79,9 +83,18 @@ class V8_EXPORT_PRIVATE AccessorAssembler : public CodeStubAssembler {
   void GenerateStoreInArrayLiteralICBaseline();
 
   void TryProbeStubCache(StubCache* stub_cache,
-                         TNode<Object> lookup_start_object, TNode<Name> name,
+                         TNode<Object> lookup_start_object,
+                         TNode<Map> lookup_start_object_map, TNode<Name> name,
                          Label* if_handler, TVariable<MaybeObject>* var_handler,
                          Label* if_miss);
+  void TryProbeStubCache(StubCache* stub_cache,
+                         TNode<Object> lookup_start_object, TNode<Name> name,
+                         Label* if_handler, TVariable<MaybeObject>* var_handler,
+                         Label* if_miss) {
+    return TryProbeStubCache(stub_cache, lookup_start_object,
+                             LoadReceiverMap(lookup_start_object), name,
+                             if_handler, var_handler, if_miss);
+  }
 
   TNode<IntPtrT> StubCachePrimaryOffsetForTesting(TNode<Name> name,
                                                   TNode<Map> map) {
@@ -96,14 +109,18 @@ class V8_EXPORT_PRIVATE AccessorAssembler : public CodeStubAssembler {
     LoadICParameters(
         TNode<Context> context, TNode<Object> receiver, TNode<Object> name,
         TNode<TaggedIndex> slot, TNode<HeapObject> vector,
-        base::Optional<TNode<Object>> lookup_start_object = base::nullopt)
+        base::Optional<TNode<Object>> lookup_start_object = base::nullopt,
+        base::Optional<TNode<TaggedIndex>> enum_index = base::nullopt,
+        base::Optional<TNode<Object>> cache_type = base::nullopt)
         : context_(context),
           receiver_(receiver),
           name_(name),
           slot_(slot),
           vector_(vector),
           lookup_start_object_(lookup_start_object ? lookup_start_object.value()
-                                                   : receiver) {}
+                                                   : receiver),
+          enum_index_(enum_index),
+          cache_type_(cache_type) {}
 
     LoadICParameters(const LoadICParameters* p, TNode<Object> unique_name)
         : context_(p->context_),
@@ -121,6 +138,8 @@ class V8_EXPORT_PRIVATE AccessorAssembler : public CodeStubAssembler {
     TNode<Object> lookup_start_object() const {
       return lookup_start_object_.value();
     }
+    TNode<TaggedIndex> enum_index() const { return *enum_index_; }
+    TNode<Object> cache_type() const { return *cache_type_; }
 
     // Usable in cases where the receiver and the lookup start object are
     // expected to be the same, i.e., when "receiver != lookup_start_object"
@@ -130,6 +149,8 @@ class V8_EXPORT_PRIVATE AccessorAssembler : public CodeStubAssembler {
       return receiver_;
     }
 
+    bool IsEnumeratedKeyedLoad() const { return enum_index_ != base::nullopt; }
+
    private:
     TNode<Context> context_;
     TNode<Object> receiver_;
@@ -137,6 +158,8 @@ class V8_EXPORT_PRIVATE AccessorAssembler : public CodeStubAssembler {
     TNode<TaggedIndex> slot_;
     TNode<HeapObject> vector_;
     base::Optional<TNode<Object>> lookup_start_object_;
+    base::Optional<TNode<TaggedIndex>> enum_index_;
+    base::Optional<TNode<Object>> cache_type_;
   };
 
   struct LazyLoadICParameters {
@@ -246,6 +269,11 @@ class V8_EXPORT_PRIVATE AccessorAssembler : public CodeStubAssembler {
       return IsDefineNamedOwn() || IsDefineKeyedOwn();
     }
 
+    StubCache* stub_cache(Isolate* isolate) const {
+      return IsAnyDefineOwn() ? isolate->define_own_stub_cache()
+                              : isolate->store_stub_cache();
+    }
+
    private:
     TNode<Context> context_;
     base::Optional<TNode<Object>> receiver_;
@@ -275,9 +303,9 @@ class V8_EXPORT_PRIVATE AccessorAssembler : public CodeStubAssembler {
                                              Label* miss,
                                              StoreTransitionMapFlags flags);
 
-  // Updates flags on |dict| if |name| is an interesting symbol.
-  void UpdateMayHaveInterestingSymbol(TNode<PropertyDictionary> dict,
-                                      TNode<Name> name);
+  // Updates flags on |dict| if |name| is an interesting property.
+  void UpdateMayHaveInterestingProperty(TNode<PropertyDictionary> dict,
+                                        TNode<Name> name);
 
   void JumpIfDataProperty(TNode<Uint32T> details, Label* writable,
                           Label* readonly);
@@ -334,7 +362,6 @@ class V8_EXPORT_PRIVATE AccessorAssembler : public CodeStubAssembler {
 
   void KeyedLoadIC(const LoadICParameters* p, LoadAccessMode access_mode);
   void KeyedLoadICGeneric(const LoadICParameters* p);
-  void KeyedLoadICGeneric_StringKey(const LoadICParameters* p);
   void KeyedLoadICPolymorphicName(const LoadICParameters* p,
                                   LoadAccessMode access_mode);
 
@@ -376,6 +403,10 @@ class V8_EXPORT_PRIVATE AccessorAssembler : public CodeStubAssembler {
                       TNode<TaggedIndex> slot, Label* miss,
                       ExitPoint* exit_point);
 
+  void TryEnumeratedKeyedLoad(const LoadICParameters* p,
+                              TNode<Map> lookup_start_object_map,
+                              ExitPoint* exit_point);
+
   // LoadIC implementation.
   void HandleLoadICHandlerCase(
       const LazyLoadICParameters* p, TNode<MaybeObject> handler, Label* miss,
@@ -406,7 +437,7 @@ class V8_EXPORT_PRIVATE AccessorAssembler : public CodeStubAssembler {
                                   ExitPoint* exit_point);
 
   void HandleLoadAccessor(const LazyLoadICParameters* p,
-                          TNode<CallHandlerInfo> call_handler_info,
+                          TNode<FunctionTemplateInfo> function_template_info,
                           TNode<Word32T> handler_word,
                           TNode<DataHandler> handler,
                           TNode<Uint32T> handler_kind, ExitPoint* exit_point);
@@ -469,8 +500,8 @@ class V8_EXPORT_PRIVATE AccessorAssembler : public CodeStubAssembler {
   // StoreIC implementation.
 
   void HandleStoreICProtoHandler(const StoreICParameters* p,
-                                 TNode<StoreHandler> handler, Label* miss,
-                                 ICMode ic_mode,
+                                 TNode<StoreHandler> handler, Label* slow,
+                                 Label* miss, ICMode ic_mode,
                                  ElementSupport support_elements);
   void HandleStoreICSmiHandlerCase(TNode<Word32T> handler_word,
                                    TNode<JSObject> holder, TNode<Object> value,
@@ -607,12 +638,12 @@ class ExitPoint {
   }
 
   template <class... TArgs>
-  void ReturnCallStub(Callable const& callable, TNode<Context> context,
-                      TArgs... args) {
+  void ReturnCallBuiltin(Builtin builtin, TNode<Context> context,
+                         TArgs... args) {
     if (IsDirect()) {
-      asm_->TailCallStub(callable, context, args...);
+      asm_->TailCallBuiltin(builtin, context, args...);
     } else {
-      indirect_return_handler_(asm_->CallStub(callable, context, args...));
+      indirect_return_handler_(asm_->CallBuiltin(builtin, context, args...));
     }
   }
 
