@@ -7,6 +7,8 @@
 #include "../../third_party/inspector_protocol/crdtp/cbor.h"
 #include "../../third_party/inspector_protocol/crdtp/dispatch.h"
 #include "../../third_party/inspector_protocol/crdtp/json.h"
+#include "include/v8-context.h"
+#include "include/v8-microtask-queue.h"
 #include "src/base/logging.h"
 #include "src/base/macros.h"
 #include "src/inspector/injected-script.h"
@@ -166,20 +168,6 @@ V8InspectorSessionImpl::~V8InspectorSessionImpl() {
   m_inspector->disconnect(this);
 }
 
-std::unique_ptr<V8InspectorSession::CommandLineAPIScope>
-V8InspectorSessionImpl::initializeCommandLineAPIScope(int executionContextId) {
-  auto scope =
-      std::make_unique<InjectedScript::ContextScope>(this, executionContextId);
-  auto result = scope->initialize();
-  if (!result.IsSuccess()) {
-    return nullptr;
-  }
-
-  scope->installCommandLineAPI();
-
-  return scope;
-}
-
 protocol::DictionaryValue* V8InspectorSessionImpl::agentState(
     const String16& name) {
   protocol::DictionaryValue* state = m_state->getObject(name);
@@ -335,9 +323,10 @@ V8InspectorSessionImpl::wrapObject(v8::Local<v8::Context> context,
   findInjectedScript(InspectedContext::contextId(context), injectedScript);
   if (!injectedScript) return nullptr;
   std::unique_ptr<protocol::Runtime::RemoteObject> result;
-  injectedScript->wrapObject(
-      value, groupName,
-      generatePreview ? WrapMode::kWithPreview : WrapMode::kNoPreview, &result);
+  injectedScript->wrapObject(value, groupName,
+                             generatePreview ? WrapOptions({WrapMode::kPreview})
+                                             : WrapOptions({WrapMode::kIdOnly}),
+                             &result);
   return result;
 }
 
@@ -511,6 +500,42 @@ void V8InspectorSessionImpl::triggerPreciseCoverageDeltaUpdate(
   m_profilerAgent->triggerPreciseCoverageDeltaUpdate(toString16(occasion));
 }
 
+V8InspectorSession::EvaluateResult V8InspectorSessionImpl::evaluate(
+    v8::Local<v8::Context> context, StringView expression,
+    bool includeCommandLineAPI) {
+  v8::EscapableHandleScope handleScope(m_inspector->isolate());
+  InjectedScript::ContextScope scope(this,
+                                     InspectedContext::contextId(context));
+  if (!scope.initialize().IsSuccess()) {
+    return {EvaluateResult::ResultType::kNotRun, v8::Local<v8::Value>()};
+  }
+
+  // Temporarily allow eval.
+  scope.allowCodeGenerationFromStrings();
+  scope.setTryCatchVerbose();
+  if (includeCommandLineAPI) {
+    scope.installCommandLineAPI();
+  }
+  v8::MaybeLocal<v8::Value> maybeResultValue;
+  {
+    v8::MicrotasksScope microtasksScope(scope.context(),
+                                        v8::MicrotasksScope::kRunMicrotasks);
+    const v8::Local<v8::String> source =
+        toV8String(m_inspector->isolate(), expression);
+    maybeResultValue = v8::debug::EvaluateGlobal(
+        m_inspector->isolate(), source, v8::debug::EvaluateGlobalMode::kDefault,
+        /*repl_mode=*/false);
+  }
+
+  if (scope.tryCatch().HasCaught()) {
+    return {EvaluateResult::ResultType::kException,
+            handleScope.Escape(scope.tryCatch().Exception())};
+  }
+  v8::Local<v8::Value> result;
+  CHECK(maybeResultValue.ToLocal(&result));
+  return {EvaluateResult::ResultType::kSuccess, handleScope.Escape(result)};
+}
+
 void V8InspectorSessionImpl::stop() { m_debuggerAgent->stop(); }
 
 }  // namespace v8_inspector
