@@ -9,8 +9,8 @@ const {
   ArrayPrototypePop,
   ArrayPrototypePush,
   ArrayPrototypeReverse,
-  ArrayPrototypeSplice,
   ArrayPrototypeShift,
+  ArrayPrototypeSplice,
   ArrayPrototypeUnshift,
   DateNow,
   FunctionPrototypeCall,
@@ -21,6 +21,7 @@ const {
   NumberIsFinite,
   ObjectSetPrototypeOf,
   RegExpPrototypeExec,
+  SafeStringIterator,
   StringPrototypeCodePointAt,
   StringPrototypeEndsWith,
   StringPrototypeRepeat,
@@ -29,7 +30,6 @@ const {
   StringPrototypeTrim,
   Symbol,
   SymbolAsyncIterator,
-  SafeStringIterator,
 } = primordials;
 
 const {
@@ -47,7 +47,11 @@ const {
   validateString,
   validateUint32,
 } = require('internal/validators');
-const { SymbolDispose, kEmptyObject } = require('internal/util');
+const {
+  assignFunctionName,
+  kEmptyObject,
+  SymbolDispose,
+} = require('internal/util');
 const {
   inspect,
   getStringWidth,
@@ -265,7 +269,7 @@ function InterfaceConstructor(input, output, completer, terminal) {
 
   function onkeypress(s, key) {
     self[kTtyWrite](s, key);
-    if (key && key.sequence) {
+    if (key?.sequence) {
       // If the key.sequence is half of a surrogate pair
       // (>= 0xd800 and <= 0xdfff), refresh the line so
       // the character is displayed appropriately.
@@ -344,12 +348,8 @@ ObjectSetPrototypeOf(InterfaceConstructor.prototype, EventEmitter.prototype);
 ObjectSetPrototypeOf(InterfaceConstructor, EventEmitter);
 
 class Interface extends InterfaceConstructor {
-  // eslint-disable-next-line no-useless-constructor
-  constructor(input, output, completer, terminal) {
-    super(input, output, completer, terminal);
-  }
   get columns() {
-    if (this.output && this.output.columns) return this.output.columns;
+    if (this.output?.columns) return this.output.columns;
     return Infinity;
   }
 
@@ -624,6 +624,12 @@ class Interface extends InterfaceConstructor {
 
   [kInsertString](c) {
     this[kBeforeEdit](this.line, this.cursor);
+    if (!this.isCompletionEnabled) {
+      this.line += c;
+      this.cursor += c.length;
+      this[kWriteToOutput](c);
+      return;
+    }
     if (this.cursor < this.line.length) {
       const beg = StringPrototypeSlice(this.line, 0, this.cursor);
       const end = StringPrototypeSlice(
@@ -1058,7 +1064,7 @@ class Interface extends InterfaceConstructor {
   // Handle a write from the tty
   [kTtyWrite](s, key) {
     const previousKey = this[kPreviousKey];
-    key = key || kEmptyObject;
+    key ||= kEmptyObject;
     this[kPreviousKey] = key;
 
     if (!key.meta || key.name !== 'y') {
@@ -1370,6 +1376,9 @@ class Interface extends InterfaceConstructor {
     return this[kLineObjectStream];
   }
 }
+Interface.prototype[SymbolDispose] = assignFunctionName(SymbolDispose, function() {
+  this.close();
+});
 
 module.exports = {
   Interface,
